import requests


def main():
    """
    Define this function to get your results in the form of a dictionary and return it.
    """

    targets = {}

    """
    Your awesome logic goes here

    example:
    targets = requests.get(f"http://10.10.0.1:8081/flagIds").json()
    """

    return targets



import re
import os
import time

from .logger import logger


def launchAttack(exploit, target_ip: str):
    start = time.time()
    flags_iterable = exploit(target_ip)
    if isinstance(flags_iterable, str):
        flags_iterable: list[str] = [flags_iterable]
    # for flag in flags_iterable:
    find_flags(flags_iterable)

    elapsed = time.time() - start

    logger.info(f"{exploit.__module__} took {elapsed:.2f} seconds")
    return


def find_flags(flags: list[str]):
    valid_flags = []

    if isinstance(flags, str):
        flags = [
            flags,
        ]
    print(os.environ["FLAG_REGEX"])
    if os.environ["FLAG_REGEX"] != "None":
        for flag in flags:
            m = re.finditer(pattern=os.environ["FLAG_REGEX"], string=flag)
            if m:
                for f in m:
                    valid_flags.append(f.group())

        if valid_flags:
            logger.info(f"Found valid flags: {valid_flags}")
        else:
            logger.info(f"Found no valid flags")
    else:
        logger.info(f"Returned:\n{flags}")



import os
import sys
import click
import importlib

from pathlib import Path
from threading import Thread

from .exploits import launchAttack

with open(Path(Path(__file__).parent, "exploit_template.py")) as fs:
    exploit_template = fs.read()

with open(Path(Path(__file__).parent, "target_template.py")) as fs:
    target_template = fs.read()


@click.group()
@click.option("--logs-dir", "logs", type=click.STRING, envvar="MOREL_LOGS_DIR")
@click.option(
    "--flag-regex",
    "regex",
    type=click.STRING,
    envvar="FLAG_REGEX",
)
def app(logs, regex):
    """Simple program to aid in the development and testing of exploits."""
    if "MOREL_LOGS_DIR" not in os.environ and logs is None:
        os.environ["MOREL_LOGS_DIR"] = "None"
    if "FLAG_REGEX" not in os.environ and regex is None:
        os.environ["FLAG_REGEX"] = "None"
        click.echo("You may want to set the environment variable FLAG_REGEX")


@app.command()
@click.argument("filename", required=False, type=click.Path())
@click.option("--target", "-t", is_flag=True)
def template(filename, target):
    """Generate a template for your exploit that is compatible with M1lkman"""
    confirmed = True

    chosen_template = "target" if target else "exploit"

    if not filename:
        click.echo("No filename specified")
        filename = "exploit_template.py"

    if Path(filename).exists():
        confirmed = click.confirm(
            f"File {filename} already exists. Do you want to overwrite it?",
            default=False,
            show_default=True,
        )

    if confirmed:
        click.echo(f"Generating file {filename} from {chosen_template} template")
        with open(filename, "w") as fs:
            fs.write(target_template) if target else fs.write(exploit_template)


@app.command()
@click.option(
    "--target",
    "-t",
    type=click.STRING,
    default="0.0.0.0",
    show_default=True,
    prompt="Target IP",
)
@click.argument("filename", required=True, type=click.Path(exists=True))
def test(target, filename):
    """Test your exploit against a target ip"""
    if os.system("ping -c 1 " + target + " > /dev/null") != 0:
        click.echo(f"Host {target} is unreachable")
        return
    click.echo(f"Testing {filename} on {target}")
    filename = Path(filename)
    name = filename.stem

    try:
        spec = importlib.util.spec_from_file_location(name, filename.resolve())
        module = importlib.util.module_from_spec(spec)  # type: ignore
        sys.modules[name] = module
        spec.loader.exec_module(module)  # type: ignore
        exploitfun = getattr(sys.modules[name], "main")
    except Exception as e:
        click.echo("There was an error import your exploit. Please check the syntax.")
        return

    t = Thread(
        target=launchAttack,
        args=(
            exploitfun,
            target,
        ),
        name=f"{exploitfun.__module__}_{target}",
    )
    t.start()
    t.join()


@app.command()
@click.argument("shell", type=click.Choice(["bash", "zsh", "fish"]))
def complete(shell):
    match shell:
        case "fish":
            cmdstring = "_MOREL_COMPLETE=fish_source morel > ~/.config/fish/completions/morel.fish"

        case "bash":
            cmdstring = (
                r"echo eval \"\$\(_MOREL_COMPLETE=bash_source morel\)\" >> ~/.bashrc"
            )

        case "zsh":
            cmdstring = (
                r"echo eval \"\$\(_MOREL_COMPLETE=zsh_source morel\)\" >> ~/.zshrc"
            )

        case _:
            click.echo("Error! How did you input this?")
    try:
        confirm = click.confirm(f"Executing `{cmdstring}`")
    except click.Abort:
        confirm = False
    if confirm:
        os.system(cmdstring)



from typing import Iterable
from morel import logger, Targets

log = logger.bind(file=f"{__name__}.log")


def main(target_ip: str) -> Iterable[str] | str:
    # log.debug(f"{__name__} against {target_ip}")

    flags = []

    """
    Your awesome exploit here...
    """

    return flags
    """
    The return value can be a string or any iterable of strings. You can also use yield (but make sure that you fully understand what it does and why yo're using it if you do - in most cases it might result in losing some flags).
    Despite the name you don't have to return strictly flags. The program will search for flags inside the returned value(s) using the provided regex rule.
    """



import importlib
import _collections_abc
import json
from multiprocessing import Lock
import sys
from collections import UserDict
from pathlib import Path
from typing import Iterable

from morel.singleton import SingletonMeta


class _restrictedDict(UserDict):
    def __init__(self):
        return super().__init__()

    # def __init__(self, initialD):
    #    return super().__init__(initialD)

    # def __getitem__(self, key):
    #    return UserDict.__getitem__(self, key)

    # def __setitem__(self, key, val):
    #    UserDict.__setitem__(self, key, val)

    def append(self, dict2) -> None:  # inplace
        dict2 = _restrictedDict(dict2)
        for k, v in dict2.items():
            if k not in self:
                self[k] = v
                continue

            if isinstance(self[k], dict) and not isinstance(self[k], _restrictedDict):
                self[k] = _restrictedDict(self[k])

            if isinstance(self[k], list):
                self[k] = set(self[k])

            if not isinstance(self[k], Iterable):
                self[k] = {
                    self[k],
                }

            if isinstance(self[k], _restrictedDict):
                self[k].append(dict2[k])

            elif isinstance(self[k], set):
                self[k] |= {
                    v,
                }

            # print(self)


class _Targets(_restrictedDict):
    def __init__(self):
        super().__init__()
        self._lock = Lock()
        self.update()

    # def __getitem__(self, key):
    #    return dict.__getitem__(self, key)

    def setBaseDir(self, dir):
        self.p = Path(dir)

    def add_targets(self, targets):
        self.append(targets)

    def update(self):
        if not hasattr(self, "p"):
            print(
                "Error! Targets not updated - you need to set base directory with Targets.setBaseDir(targets_directory)"
            )
            return
        else:
            print(f"basedir is {self.p}")
        with self._lock:
            files = list(self.p.glob("**/[!_]*.py"))
            targets_functions = []

            for targ in files:
                name = targ.stem

                try:
                    spec = importlib.util.spec_from_file_location(name, targ.resolve())
                    module = importlib.util.module_from_spec(spec)  # type: ignore
                    sys.modules[name] = module
                    spec.loader.exec_module(module)  # type: ignore
                    targetfun = getattr(sys.modules[name], "main")
                    targets_functions.append(targetfun)
                except Exception as e:
                    print(f"Exception on module {name}:\n{e}")

            for function in targets_functions:
                self.add_targets(function())


Targets = _Targets()
if __name__ == "__main__":
    json.dump(Targets, sys.stdout, indent=2)



import os
import sys

from loguru import logger

logger.remove()

_conf = {
    "handlers": [
        {
            "sink": sys.stdout,
            "enqueue": True,
            "format": "<d>{time:HH:mm:ss.SS}</d> <c>{name}</c> <level>[{level:^8}]</level> : {message}",
            "level": "DEBUG",
            "colorize": True,
        }
    ]
}


logger.configure(**_conf)
logger = logger.bind(file="")
